#version 150 core
#extension GL_EXT_gpu_shader4 : enable
#extension GL_EXT_texture_array : enable

precision highp float;
precision highp int;

uniform sampler2D uTex;
uniform int uScale;
uniform float uSigma = 1.125;
uniform float uThreshold = 0.05125;

out vec4 outCoarse;
out vec4 outDetail;

void main()
{
	const float b3spline[25] = float[25](
		1.0/256.0, 1.0/64.0, 3.0/128.0, 1.0/64.0, 1.0/256.0,
		1.0/64.0,  1.0/16.0, 3.0/32.0,  1.0/16.0, 1.0/64.0,
		3.0/128.0, 3.0/32.0, 9.0/64.0,  3.0/32.0, 3.0/128.0,
		1.0/64.0,  1.0/16.0, 3.0/32.0,  1.0/16.0, 1.0/64.0,
		1.0/256.0, 1.0/64.0, 3.0/128.0, 1.0/64.0, 1.0/256.0 );

	const ivec2 offsets[25] = ivec2[25]( 
		ivec2(-2,-2), ivec2(-1,-2), ivec2(0,-2), ivec2(1,-2), ivec2(2,-2), 
		ivec2(-2,-1), ivec2(-1,-1), ivec2(0,-2), ivec2(1,-1), ivec2(2,-1), 
		ivec2(-2, 0), ivec2(-1, 0), ivec2(0, 0), ivec2(1, 0), ivec2(2, 0), 
		ivec2(-2, 1), ivec2(-1, 1), ivec2(0, 1), ivec2(1, 1), ivec2(2, 1),
		ivec2(-2, 2), ivec2(-1, 2), ivec2(0, 2), ivec2(1, 2), ivec2(2, 2) );

	vec4 sum = vec4(0.0);
	float weightSum = 0.0;
	ivec2 centerPos = ivec2(gl_FragCoord.xy);
	vec4 center = texelFetch(uTex, ivec2(gl_FragCoord.xy), 0);
	
    for(int i=0; i<25; i++)
    {
		vec4 sample = texelFetch(uTex, centerPos + offsets[i]*uScale, 0);	
		
		vec4 delta = sample - center;
		float weight = exp(dot(delta, delta) / uSigma) * b3spline[i];
		
		sum += weight * sample;
		weightSum += weight;
	}

	vec4 coarse = sum / weightSum;
	vec4 detail = center - coarse;

	outCoarse = coarse;
	outDetail = max(vec4(0.0), abs(detail) - vec4(uThreshold)) * sign(detail);
}